#include "config.h"
#include "tuneparams.h"

#include "dvbtuneparamscommon.h"
#include "tune.h"

#include <map>
#include <iostream>

// transTable is an auxillary mapping from
// string keywords (eg FEC_1_2) to numbers
// FEC_1_2. Used in parsing. See translate() and
// DVBTTuneParams::createFromTZAP() 

typedef std::map<std::string, int> StringIntMap;

static StringIntMap transTable;
static bool transTableSetup = false;

int translateTuneParam(const std::string &t) 
{
  if (!transTableSetup) {
    transTable["INVERSION_AUTO"] = INVERSION_AUTO;
    transTable["INVERSION_OFF"] = INVERSION_OFF;
    transTable["INVERSION_ON"] = INVERSION_ON;
    transTable["FEC_NONE"] = FEC_NONE;
    transTable["FEC_1_2"] = FEC_1_2;
    transTable["FEC_2_3"] = FEC_2_3;
    transTable["FEC_3_4"] = FEC_3_4;
    transTable["QAM_16"] = QAM_16;
    transTable["QAM_32"] = QAM_32;
    transTable["QAM_64"] = QAM_64;
    transTable["GUARD_INTERVAL_1_4"] = GUARD_INTERVAL_1_4;
    transTable["GUARD_INTERVAL_1_8"] = GUARD_INTERVAL_1_8;
    transTable["GUARD_INTERVAL_1_16"] = GUARD_INTERVAL_1_16;
    transTable["GUARD_INTERVAL_1_32"] = GUARD_INTERVAL_1_32;
    transTable["HIERARCHY_NONE"] = HIERARCHY_NONE;
    transTable["BANDWIDTH_7_MHZ"] = BANDWIDTH_7_MHZ;
    transTable["BANDWIDTH_8_MHZ"] = BANDWIDTH_8_MHZ;
    transTable["TRANSMISSION_MODE_2K"] = TRANSMISSION_MODE_2K;
    transTable["TRANSMISSION_MODE_8K"] = TRANSMISSION_MODE_8K;
    transTableSetup = true;
  }

  StringIntMap::const_iterator i = transTable.find(t);
  if (i == transTable.end()) {
    std::cerr << "Fatal Error: no translation for \"" << t << "\" in " << __FILE__ << ":" << __LINE__ << "\n";
    exit(1);
  }
  return i->second;
}
