/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#include "config.h"
#include "source.h"

#include <climits>
#include <iostream>
#include <cstdlib>

Priority::~Priority()
{
}

void Priority::setPriority( int newPriority )
{
  std::cerr << "Fatal error: attempt to call unimplemented setPriority method\n";
  exit(1);
 }

Sink::~Sink()
{
}

void Sink::notifyUnsubscribe(Source *s)
{
}

Source::~Source()
{
}

bool Source::subscribe(Sink *s)
{
  subscribers.push_back(s);
  return true;
}

void Source::unsubscribe(Sink *s)
{
  subscribers.remove(s);
  s->notifyUnsubscribe(this);
}

void Source::unsubscribeAll()
{
  SinkList copy = subscribers;

  for (SinkList::const_iterator i = copy.begin(); i != copy.end(); i++)
    unsubscribe(*i);
}

void Source::sendData(unsigned char *data, unsigned size) const
{
  for (SinkList::const_iterator i = subscribers.begin(); i != subscribers.end(); i++)
    (*i)->receiveData(data, size);
}

int Source::unsubscribe( int priority )
{
  // Kick subscribers with priority < specified priority
  // Return number of subscribers kicked.
  int count = 0;
  SinkList copy = subscribers;

  for (SinkList::iterator i = copy.begin(); i != copy.end(); i++) {
    Sink *d = *i;

    if (d->getPriority() < priority) {
      unsubscribe(d);
      count++;
    }
  }
  return count;
}

int Source::getNumSubscribers(int priority) const
{
  // Return the number of subscribers with priority >= specified priority
  int count = 0;
  for (SinkList::const_iterator i = subscribers.begin(); i != subscribers.end(); i++) {
    const Sink *d = *i;

    if (d->getPriority() >= priority)
      count++;
  }
  return count;
}

int Source::getPriority() const
{
  // Return the maximum subscribed priority
  int result = INT_MIN;
  for (SinkList::const_iterator i = subscribers.begin(); i != subscribers.end(); i++) {
    const Sink *c = *i;

    if (c->getPriority() > result)
      result = c->getPriority();
  }
  return result;
}
